@@ -1,5 +1,5 @@
 #------------------------------------------------------------------------------
-# Copyright (c) 2020, 2022, Oracle and/or its affiliates.
+# Copyright (c) 2020, 2023, Oracle and/or its affiliates.
 #
 # This software is dual-licensed to you under the Universal Permissive License
 # (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl and Apache License
@@ -68,8 +68,6 @@ class TestCase(test_env.BaseTestCase):
         count, = self.cursor.fetchone()
         self.assertEqual(count, len(rows))
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3202_bind_plsql_boolean_collection_in(self):
         "3202 - test binding a boolean collection (in)"
         type_obj = self.connection.gettype("PKG_TESTBOOLEANS.UDT_BOOLEANLIST")
@@ -80,8 +78,6 @@ class TestCase(test_env.BaseTestCase):
                                       (obj,))
         self.assertEqual(result, 5)
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3203_bind_plsql_boolean_collection_out(self):
         "3203 - test binding a boolean collection (out)"
         type_obj = self.connection.gettype("PKG_TESTBOOLEANS.UDT_BOOLEANLIST")
@@ -89,8 +85,6 @@ class TestCase(test_env.BaseTestCase):
         self.cursor.callproc("pkg_TestBooleans.TestOutArrays", (6, obj))
         self.assertEqual(obj.aslist(), [True, False, True, False, True, False])
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3204_bind_plql_date_collection_in(self):
         "3204 - test binding a PL/SQL date collection (in)"
         type_obj = self.connection.gettype("PKG_TESTDATEARRAYS.UDT_DATELIST")
@@ -103,8 +97,6 @@ class TestCase(test_env.BaseTestCase):
                                       (2, datetime.datetime(2016, 2, 1), obj))
         self.assertEqual(result, 24.75)
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3205_bind_plqsl_date_collection_in_out(self):
         "3205 - test binding a PL/SQL date collection (in/out)"
         type_obj = self.connection.gettype("PKG_TESTDATEARRAYS.UDT_DATELIST")
@@ -122,8 +114,6 @@ class TestCase(test_env.BaseTestCase):
         ]
         self.assertEqual(obj.aslist(), expected_values)
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3206_bind_plsql_date_collection_out(self):
         "3206 - test binding a PL/SQL date collection (out)"
         type_obj = self.connection.gettype("PKG_TESTDATEARRAYS.UDT_DATELIST")
@@ -136,8 +126,6 @@ class TestCase(test_env.BaseTestCase):
         ]
         self.assertEqual(obj.aslist(), expected_values)
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3207_bind_plsql_number_collection_in(self):
         "3207 - test binding a PL/SQL number collection (in)"
         type_name = "PKG_TESTNUMBERARRAYS.UDT_NUMBERLIST"
@@ -149,8 +137,6 @@ class TestCase(test_env.BaseTestCase):
                                       (5, obj))
         self.assertEqual(result, 155)
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3208_bind_plsql_number_collection_in_out(self):
         "3208 - test binding a PL/SQL number collection (in/out)"
         type_name = "PKG_TESTNUMBERARRAYS.UDT_NUMBERLIST"
@@ -161,8 +147,6 @@ class TestCase(test_env.BaseTestCase):
         self.cursor.callproc("pkg_TestNumberArrays.TestInOutArrays", (4, obj))
         self.assertEqual(obj.aslist(), [50, 80, 30, 20])
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3209_bind_plsql_number_collection_out(self):
         "3209 - test binding a PL/SQL number collection (out)"
         type_name = "PKG_TESTNUMBERARRAYS.UDT_NUMBERLIST"
@@ -171,8 +155,6 @@ class TestCase(test_env.BaseTestCase):
         self.cursor.callproc("pkg_TestNumberArrays.TestOutArrays", (3, obj))
         self.assertEqual(obj.aslist(), [100, 200, 300])
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3210_bind_plsql_record_array(self):
         "3210 - test binding an array of PL/SQL records (in)"
         rec_type = self.connection.gettype("PKG_TESTRECORDS.UDT_RECORD")
@@ -204,8 +186,6 @@ class TestCase(test_env.BaseTestCase):
                          "to_timestamp('2017-01-03 00:00:00', " \
                          "'YYYY-MM-DD HH24:MI:SS'), false, 10, 4)")
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3211_bind_plsql_record_in(self):
         "3211 - test binding a PL/SQL record (in)"
         type_obj = self.connection.gettype("PKG_TESTRECORDS.UDT_RECORD")
@@ -225,8 +205,6 @@ class TestCase(test_env.BaseTestCase):
                          "to_timestamp('2016-02-12 14:25:36', " \
                          "'YYYY-MM-DD HH24:MI:SS'), false, 21, 5)")
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3212_bind_plsql_record_out(self):
         "3212 - test binding a PL/SQL record (out)"
         type_obj = self.connection.gettype("PKG_TESTRECORDS.UDT_RECORD")
@@ -248,8 +226,6 @@ class TestCase(test_env.BaseTestCase):
         self.assertEqual(obj.PLSINTEGERVALUE, 45)
         self.assertEqual(obj.BINARYINTEGERVALUE, 10)
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3213_bind_plsql_string_collection_in(self):
         "3213 - test binding a PL/SQL string collection (in)"
         type_name = "PKG_TESTSTRINGARRAYS.UDT_STRINGLIST"
@@ -262,8 +238,6 @@ class TestCase(test_env.BaseTestCase):
                                       (5, obj))
         self.assertEqual(result, 45)
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3214_bind_plsql_string_collection_in_out(self):
         "3214 - test binding a PL/SQL string collection (in/out)"
         type_name = "PKG_TESTSTRINGARRAYS.UDT_STRINGLIST"
@@ -280,8 +254,6 @@ class TestCase(test_env.BaseTestCase):
         ]
         self.assertEqual(obj.aslist(), expected_values)
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3215_bind_plsql_string_collection_out(self):
         "3215 - test binding a PL/SQL string collection (out)"
         type_name = "PKG_TESTSTRINGARRAYS.UDT_STRINGLIST"
@@ -296,8 +268,6 @@ class TestCase(test_env.BaseTestCase):
         ]
         self.assertEqual(obj.aslist(), expected_values)
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_3216_bind_plsql_string_collection_out_with_holes(self):
         "3216 - test binding a PL/SQL string collection (out with holes)"
         type_name = "PKG_TESTSTRINGARRAYS.UDT_STRINGLIST"
@@ -542,5 +512,43 @@ class TestCase(test_env.BaseTestCase):
         self.cursor.executemany(sql, rows, batcherrors=True)
         self.assertEqual(self.cursor.getbatcherrors(), [])
 
+    def test_3227_batch_error_multiple_execute(self):
+        "3227 - test batcherrors mode with multiple executes"
+        self.cursor.execute("truncate table TestArrayDML")
+        rows_1 = [
+            (1, "Value 1", 100),
+            (2, "Value 2", 200),
+            (2, "Value 2", 200)
+        ]
+        rows_2 = [
+            (3, "Value 3", 300),
+            (3, "Value 3", 300),
+            (4, "Value 4", 400)
+        ]
+        sql = """
+                insert into TestArrayDML (IntCol, StringCol, IntCol2)
+                values (:1, :2, :3)"""
+        self.cursor.executemany(sql, rows_1, batcherrors=True)
+        expected_errors = [(2, "ORA-00001")]
+        actual_errors = [(e.offset, e.full_code) \
+                         for e in self.cursor.getbatcherrors()]
+        self.assertEqual(actual_errors, expected_errors)
+        self.cursor.executemany(sql, rows_2, batcherrors=True)
+        expected_errors = [(1, "ORA-00001")]
+        actual_errors = [(e.offset, e.full_code) \
+                         for e in self.cursor.getbatcherrors()]
+        self.assertEqual(actual_errors, expected_errors)
+
+    def test_3228_record_based_on_table_type(self):
+        "3228 - test %ROWTYPE record type"
+        type_obj = self.connection.gettype("TESTTEMPTABLE%ROWTYPE")
+        self.assertEqual(type_obj.attributes[3].name, "NUMBERCOL")
+
+    def test_3229_collection_of_records_based_on_table_type(self):
+        "3229 - test collection of %ROWTYPE record type"
+        type_name = "PKG_TESTBINDOBJECT.UDT_COLLECTIONROWTYPE"
+        type_obj = self.connection.gettype(type_name)
+        self.assertEqual(type_obj.element_type.attributes[3].name, "NUMBERCOL")
+
 if __name__ == "__main__":
     test_env.run_test_cases()
