@@ -1,5 +1,5 @@
 #------------------------------------------------------------------------------
-# Copyright (c) 2020, 2022, Oracle and/or its affiliates.
+# Copyright (c) 2020, 2023, Oracle and/or its affiliates.
 #
 # This software is dual-licensed to you under the Universal Permissive License
 # (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl and Apache License
@@ -214,8 +214,6 @@ class TestCase(test_env.BaseTestCase):
         var.setvalue(0, value_to_set)
         self.assertEqual(var.getvalue(), value_to_set)
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support database objects yet")
     def test_4319_var_type_with_object_type(self):
         "4319 - test that an object type can be used as type in cursor.var()"
         obj_type = self.connection.gettype("UDT_OBJECT")
@@ -228,17 +226,15 @@ class TestCase(test_env.BaseTestCase):
         exp = "udt_Object(28, 'Bind obj out', null, null, null, null, null)"
         self.assertEqual(result, exp)
 
-    @unittest.skipIf(test_env.get_is_thin(),
-                     "thin mode doesn't support XML type objects yet")
     def test_4320_fetch_xmltype(self):
         "4320 - test that fetching an XMLType returns a string"
         int_val = 5
         label = "IntCol"
-        expected_result = "<%s>%s</%s>" % (label, int_val, label)
-        self.cursor.execute("""
-                select XMLElement("%s", IntCol)
+        expected_result = f"<{label}>{int_val}</{label}>"
+        self.cursor.execute(f"""
+                select XMLElement("{label}", IntCol)
                 from TestStrings
-                where IntCol = :int_val""" % label,
+                where IntCol = :int_val""",
                 int_val=int_val)
         result, = self.cursor.fetchone()
         self.assertEqual(result, expected_result)
@@ -727,6 +723,7 @@ class TestCase(test_env.BaseTestCase):
         "4538 - kill connection with open cursor"
         admin_conn = test_env.get_admin_connection()
         conn = test_env.get_connection()
+        self.assertEqual(conn.is_healthy(), True)
         cursor = conn.cursor()
         cursor.execute("""
                 select
@@ -745,6 +742,7 @@ class TestCase(test_env.BaseTestCase):
         "4359 - kill connection in cursor context manager"
         admin_conn = test_env.get_admin_connection()
         conn = test_env.get_connection()
+        self.assertEqual(conn.is_healthy(), True)
         with conn.cursor() as cursor:
             cursor.execute("""
                     select
@@ -757,6 +755,7 @@ class TestCase(test_env.BaseTestCase):
                 admin_cursor.execute(sql)
             self.assertRaisesRegex(oracledb.DatabaseError, "^DPY-4011:",
                                    cursor.execute, "select user from dual")
+            self.assertEqual(conn.is_healthy(), False)
 
     def test_4360_fetchmany(self):
         "4360 - fetchmany() with and without parameters"
@@ -815,6 +814,42 @@ class TestCase(test_env.BaseTestCase):
             self.cursor.parse(sql)
             self.cursor.execute(sql, ("Updated value", data[0]))
 
+    def test_4364_binds_between_comment_blocks(self):
+        "4364 - test bindnames() for bind variables between comment blocks"
+        self.cursor.prepare("""
+                select
+                    /* comment 1 */
+                    :a,
+                    /* comment 2 */
+                    :b
+                    /* comment 3 */
+                from dual""")
+        self.assertEqual(self.cursor.bindnames(), ["A", "B"])
+
+    def test_4365_add_column_to_cached_query(self):
+        "4365 - test addition of column to cached query"
+        table_name = "test_4365"
+        try:
+            self.cursor.execute(f"drop table {table_name}")
+        except:
+            pass
+        data = ('val 1', 'val 2')
+        self.cursor.execute(f"create table {table_name} (col1 varchar2(10))")
+        self.cursor.execute(f"insert into {table_name} values (:1)", [data[0]])
+        self.connection.commit()
+        self.cursor.execute(f"select * from {table_name}")
+        self.assertEqual(self.cursor.fetchall(), [(data[0],)])
+        self.cursor.execute(f"alter table {table_name} add col2 varchar2(10)")
+        self.cursor.execute(f"update {table_name} set col2 = :1", [data[1]])
+        self.connection.commit()
+        self.cursor.execute(f"select * from {table_name}")
+        self.assertEqual(self.cursor.fetchall(), [data])
+
+    def test_4366_populate_array_var_with_too_many_elements(self):
+        "4366 - test population of array var with too many elements"
+        var = self.cursor.arrayvar(int, 3)
+        self.assertRaisesRegex(oracledb.ProgrammingError, "^DPY-2016:",
+                               var.setvalue, 0, [1, 2, 3, 4])
 
 if __name__ == "__main__":
     test_env.run_test_cases()
