@@ -1,5 +1,5 @@
 #------------------------------------------------------------------------------
-# Copyright (c) 2021, 2022, Oracle and/or its affiliates.
+# Copyright (c) 2021, 2023, Oracle and/or its affiliates.
 #
 # This software is dual-licensed to you under the Universal Permissive License
 # (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl and Apache License
@@ -478,10 +478,9 @@ class TestCase(test_env.BaseTestCase):
                                             tcp_connect_timeout=in_val)
             tcp_timeout_val = f"(TRANSPORT_CONNECT_TIMEOUT={out_val})"
             connect_string = f"(DESCRIPTION={tcp_timeout_val}" + \
-                             f"(ADDRESS_LIST=(ADDRESS=(PROTOCOL=tcp)" + \
-                             f"(HOST={host})(PORT=1521)))(CONNECT_DATA=" + \
-                             f"(SERVICE_NAME={service_name}))" + \
-                             f"(SECURITY=(SSL_SERVER_DN_MATCH=True)))"
+                             f"(ADDRESS=(PROTOCOL=tcp)" + \
+                             f"(HOST={host})(PORT=1521))(CONNECT_DATA=" + \
+                             f"(SERVICE_NAME={service_name})))"
             self.assertEqual(params.get_connect_string(), connect_string)
 
     def test_4532_multiple_alias_entry_tnsnames(self):
@@ -520,5 +519,77 @@ class TestCase(test_env.BaseTestCase):
             self.assertEqual(params.cclass, cclass)
             self.assertEqual(params.purity, purity_int)
 
+    def test_4534_connect_descriptor_small_container_first(self):
+        "4534 - test connect descriptor with different containers (small 1st)"
+        connect_string = """
+            (DESCRIPTION=
+                (ADDRESS=(PROTOCOL=tcp)(HOST=host1)(PORT=1521))
+                (ADDRESS_LIST=
+                    (ADDRESS=(PROTOCOL=tcp)(HOST=host2a)(PORT=1522))
+                    (ADDRESS=(PROTOCOL=tcp)(HOST=host2b)(PORT=1523)))
+                (ADDRESS=(PROTOCOL=tcp)(HOST=host3)(PORT=1524))
+                (CONNECT_DATA=(SERVICE_NAME=my_service_34))
+            )"""
+        params = oracledb.ConnectParams()
+        params.parse_connect_string(connect_string)
+        self.assertEqual(params.host, ["host1", "host2a", "host2b", "host3"])
+
+    def test_4535_connect_descriptor_small_container_second(self):
+        "4535 - test connect descriptor with different containers (small 2nd)"
+        connect_string = """
+            (DESCRIPTION=
+                (ADDRESS_LIST=
+                    (ADDRESS=(PROTOCOL=tcp)(HOST=host1a)(PORT=1532))
+                    (ADDRESS=(PROTOCOL=tcp)(HOST=host1b)(PORT=1533)))
+                (ADDRESS=(PROTOCOL=tcp)(HOST=host2)(PORT=1534))
+                (ADDRESS_LIST=
+                    (ADDRESS=(PROTOCOL=tcp)(HOST=host3a)(PORT=1535))
+                    (ADDRESS=(PROTOCOL=tcp)(HOST=host3b)(PORT=1536)))
+                (CONNECT_DATA=(SERVICE_NAME=my_service_34))
+            )"""
+        params = oracledb.ConnectParams()
+        params.parse_connect_string(connect_string)
+        self.assertEqual(params.host,
+                         ["host1a", "host1b", "host2", "host3a", "host3b"])
+
+    def test_4536_build_connect_string_with_source_route(self):
+        "4536 - test building connect string with source route designation"
+        host = "my_host4536"
+        service_name = "my_service4536"
+        options = [
+            ("on", True),
+            ("off", False),
+            ("true", True),
+            ("false", False),
+            ("yes", True),
+            ("no", False)
+        ]
+
+        for in_val, has_section in options:
+            connect_string = f"""
+                (DESCRIPTION=
+                    (SOURCE_ROUTE={in_val})
+                    (ADDRESS=(PROTOCOL=tcp)(HOST=host1)(PORT=1521))
+                    (ADDRESS=(PROTOCOL=tcp)(HOST=host2)(PORT=1522))
+                    (CONNECT_DATA=(SERVICE_NAME=my_service_35))
+                )"""
+            params = oracledb.ConnectParams()
+            params.parse_connect_string(connect_string)
+            source_route_clause = "(SOURCE_ROUTE=ON)" if has_section else ""
+            connect_string = \
+                    f"(DESCRIPTION={source_route_clause}" + \
+                    f"(ADDRESS_LIST=" + \
+                    f"(ADDRESS=(PROTOCOL=tcp)(HOST=host1)(PORT=1521))" + \
+                    f"(ADDRESS=(PROTOCOL=tcp)(HOST=host2)(PORT=1522)))" + \
+                    f"(CONNECT_DATA=(SERVICE_NAME=my_service_35)))"
+            self.assertEqual(params.get_connect_string(), connect_string)
+
+    def test_4537_no_connect_string(self):
+        "4537 - test connect parameters which generate no connect string"
+        params = oracledb.ConnectParams()
+        self.assertEqual(params.get_connect_string(), None)
+        params.set(mode=oracledb.SYSDBA)
+        self.assertEqual(params.get_connect_string(), None)
+
 if __name__ == "__main__":
     test_env.run_test_cases()
